/**
 * servable v0.4.1
 * https://github.com/maniator/servable#readme
 * @author Neal Lubin
 * @preserve
 */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
	typeof define === 'function' && define.amd ? define(['exports'], factory) :
	(factory((global.Servable = global.Servable || {})));
}(this, (function (exports) { 'use strict';

function _typeof(obj) {
  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _toArray(arr) {
  return _arrayWithHoles(arr) || _iterableToArray(arr) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  }
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];

  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }

  return (hint === "string" ? String : Number)(input);
}

function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");

  return typeof key === "symbol" ? key : String(key);
}

function _addElementPlacement(element, placements, silent) {
  var keys = placements[element.placement];

  if (!silent && keys.indexOf(element.key) !== -1) {
    throw new TypeError("Duplicated element (" + element.key + ")");
  }

  keys.push(element.key);
}

function _fromElementDescriptor(element) {
  var obj = {
    kind: element.kind,
    key: element.key,
    placement: element.placement,
    descriptor: element.descriptor
  };
  var desc = {
    value: "Descriptor",
    configurable: true
  };
  Object.defineProperty(obj, Symbol.toStringTag, desc);
  if (element.kind === "field") obj.initializer = element.initializer;
  return obj;
}

function _toElementDescriptors(elementObjects) {
  if (elementObjects === undefined) return;
  return _toArray(elementObjects).map(function (elementObject) {
    var element = _toElementDescriptor(elementObject);

    _disallowProperty(elementObject, "finisher", "An element descriptor");

    _disallowProperty(elementObject, "extras", "An element descriptor");

    return element;
  });
}

function _toElementDescriptor(elementObject) {
  var kind = String(elementObject.kind);

  if (kind !== "method" && kind !== "field") {
    throw new TypeError('An element descriptor\'s .kind property must be either "method" or' + ' "field", but a decorator created an element descriptor with' + ' .kind "' + kind + '"');
  }

  var key = _toPropertyKey(elementObject.key);

  var placement = String(elementObject.placement);

  if (placement !== "static" && placement !== "prototype" && placement !== "own") {
    throw new TypeError('An element descriptor\'s .placement property must be one of "static",' + ' "prototype" or "own", but a decorator created an element descriptor' + ' with .placement "' + placement + '"');
  }

  var descriptor = elementObject.descriptor;

  _disallowProperty(elementObject, "elements", "An element descriptor");

  var element = {
    kind: kind,
    key: key,
    placement: placement,
    descriptor: Object.assign({}, descriptor)
  };

  if (kind !== "field") {
    _disallowProperty(elementObject, "initializer", "A method descriptor");
  } else {
    _disallowProperty(descriptor, "get", "The property descriptor of a field descriptor");

    _disallowProperty(descriptor, "set", "The property descriptor of a field descriptor");

    _disallowProperty(descriptor, "value", "The property descriptor of a field descriptor");

    element.initializer = elementObject.initializer;
  }

  return element;
}

function _toElementFinisherExtras(elementObject) {
  var element = _toElementDescriptor(elementObject);

  var finisher = _optionalCallableProperty(elementObject, "finisher");

  var extras = _toElementDescriptors(elementObject.extras);

  return {
    element: element,
    finisher: finisher,
    extras: extras
  };
}

function _fromClassDescriptor(elements) {
  var obj = {
    kind: "class",
    elements: elements.map(_fromElementDescriptor)
  };
  var desc = {
    value: "Descriptor",
    configurable: true
  };
  Object.defineProperty(obj, Symbol.toStringTag, desc);
  return obj;
}

function _toClassDescriptor(obj) {
  var kind = String(obj.kind);

  if (kind !== "class") {
    throw new TypeError('A class descriptor\'s .kind property must be "class", but a decorator' + ' created a class descriptor with .kind "' + kind + '"');
  }

  _disallowProperty(obj, "key", "A class descriptor");

  _disallowProperty(obj, "placement", "A class descriptor");

  _disallowProperty(obj, "descriptor", "A class descriptor");

  _disallowProperty(obj, "initializer", "A class descriptor");

  _disallowProperty(obj, "extras", "A class descriptor");

  var finisher = _optionalCallableProperty(obj, "finisher");

  var elements = _toElementDescriptors(obj.elements);

  return {
    elements: elements,
    finisher: finisher
  };
}

function _disallowProperty(obj, name, objectType) {
  if (obj[name] !== undefined) {
    throw new TypeError(objectType + " can't have a ." + name + " property.");
  }
}

function _optionalCallableProperty(obj, name) {
  var value = obj[name];

  if (value !== undefined && typeof value !== "function") {
    throw new TypeError("Expected '" + name + "' to be a function");
  }

  return value;
}

/**
 * @private
 * @returns {null}
 */
var noop = function noop() {
  return null;
};
/**
 * @private
 * @param p
 * @returns {boolean}
 */

var isPromise = function isPromise(p) {
  return p !== null && _typeof(p) === 'object' && typeof p.then === 'function';
};
/**
 * @private
 * @param o$
 * @returns {boolean}
 */

var isObservable = function isObservable(o$) {
  return o$ !== null && _typeof(o$) === 'object' && typeof o$.subscribe === 'function';
};

/**
 * @param {{next: Function, error: Function, complete: Function}} observer
 */

var Observer =
/*#__PURE__*/
function () {
  function Observer() {
    var observer = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Observer);

    this.isComplete = false;
    this.dispose = noop;
    this.setupObserver(observer);
  }
  /**
   * Cleans up and disposes of all events in the stream
   *
   * @returns {Observer}
   */


  _createClass(Observer, [{
    key: "cleanup",
    value: function cleanup() {
      this.isComplete = true;
      this.dispose();
      this.setupObserverObject = {
        next: noop,
        error: noop,
        complete: noop
      };
      this.dispose = noop;
      return this;
    }
  }, {
    key: "catchErrors",
    value: function catchErrors(callback) {
      var _this = this;

      return function () {
        try {
          return callback.apply(void 0, arguments);
        } catch (errors) {
          _this.onError(errors);
        }
      };
    }
    /**
     * @param {Function} callback
     * @returns {Function}
     */

  }, {
    key: "use",
    value: function use(callback) {
      var _this2 = this;

      var callbackCatch = this.catchErrors(callback);
      var response = callbackCatch({
        next: function next(value) {
          return _this2.onNext(value);
        },
        error: function error() {
          return _this2.onError.apply(_this2, arguments);
        },
        complete: function complete() {
          return _this2.onComplete();
        }
      });

      if (typeof response === 'function') {
        this.dispose = this.catchErrors(response);
      } else {
        this.dispose = noop;
      }

      return this.dispose;
    }
  }, {
    key: "onNext",
    value: function onNext(value) {
      if (this.isComplete) {
        return null;
      }

      return this.setupObserverObject.next(value);
    }
  }, {
    key: "onError",
    value: function onError() {
      var _this$setupObserverOb;

      if (this.isComplete) {
        return null;
      }

      return (_this$setupObserverOb = this.setupObserverObject).error.apply(_this$setupObserverOb, arguments);
    }
  }, {
    key: "onComplete",
    value: function onComplete() {
      if (this.isComplete) {
        return null;
      }

      this.cleanup();
      return this.setupObserverObject.complete();
    }
  }, {
    key: "setupObserver",
    value: function setupObserver() {
      var setupObserverObject = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
        next: noop,
        error: noop,
        complete: noop
      };

      // assumes that an object was passed as first value to subscription
      if (typeof setupObserverObject.next !== 'function' && _typeof(setupObserverObject.next) === 'object') {
        return this.setupObserver(setupObserverObject.next);
      }

      var _setupObserverObject$ = setupObserverObject.next,
          next = _setupObserverObject$ === void 0 ? noop : _setupObserverObject$,
          _setupObserverObject$2 = setupObserverObject.error,
          error = _setupObserverObject$2 === void 0 ? noop : _setupObserverObject$2,
          _setupObserverObject$3 = setupObserverObject.complete,
          complete = _setupObserverObject$3 === void 0 ? noop : _setupObserverObject$3;
      this.setupObserverObject = {
        next: this.catchErrors(next),
        error: error,
        complete: this.catchErrors(complete)
      };
      return this;
    }
  }]);

  return Observer;
}();

/**
 * @param {Function} callback
 * @param {{next: Function, error: Function, complete: Function}} observer
 */

var Subscription =
/*#__PURE__*/
function () {
  function Subscription() {
    var callback = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : noop;
    var observer = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, Subscription);

    this.observer = new Observer(observer);
    this.observer.use(callback);
  }

  _createClass(Subscription, [{
    key: "unsubscribe",

    /**
     * Unsubscribe to the event stream
     *
     * @returns {void}
     */
    value: function unsubscribe() {
      this.observer.cleanup();
    }
    /**
     *
     * @returns {boolean}
     */

  }, {
    key: "isComplete",
    get: function get$$1() {
      return this.observer.isComplete;
    }
  }, {
    key: "onNext",
    get: function get$$1() {
      return this.observer.onNext;
    }
  }, {
    key: "onError",
    get: function get$$1() {
      return this.observer.onError;
    }
  }, {
    key: "onComplete",
    get: function get$$1() {
      return this.observer.onComplete;
    }
  }], [{
    key: "createSimple",
    value: function createSimple(observer) {
      return new Subscription(noop, observer);
    }
  }]);

  return Subscription;
}();

/**
 * Creates a new Observable
 * @param {Function} observerCallback
 *
 * @example
 * const obs$ = new Observable((observer) => {
 *   for (let i = 0; i < 10; ++i) {
 *     observer.next(i);
 *   }
 *
 *   observer.complete();
 * });
 *
 * @example
 * const obs$ = Observable.create((observer) => {
 *   for (let i = 0; i < 10; ++i) {
 *     observer.next(i);
 *   }
 *
 *   observer.complete();
 * });
 */

var Observable =
/*#__PURE__*/
function () {
  function Observable(observerCallback) {
    _classCallCheck(this, Observable);

    this.observerCallback = observerCallback;
  }
  /**
   * @param {Function} observerCallback
   * @returns {Observable}
   */


  _createClass(Observable, [{
    key: "subscribe",

    /**
     * Subscribe to an observable stream
     *
     * @param {{ next: Function, error: Function, complete: Function }} next
     * @param {null} error
     * @param {null} complete
     * @returns {Subscription}
     *
     * @example
     * observable$.subscribe({
     *   next(value) {
     *     console.log('logs if next', value);
     *   },
     *   error() {
     *     console.log('logs if error');
     *   },
     *   complete() {
     *     console.log('logs if complete');
     *   }
     * });
     */

    /**
     *
     * @param {Function} [next]
     * @param {Function} [error]
     * @param {Function} [complete]
     * @returns {Subscription}
     *
     * @example
     * observable$.subscribe(
     *  (value) => console.log('logs if next', value),
     *  () => console.log('logs if error'),
     *  () => console.log('logs if complete')
     * );
     */
    value: function subscribe() {
      var next = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : noop;
      var error = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : noop;
      var complete = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : noop;
      return new Subscription(this.observerCallback, {
        next: next,
        error: error,
        complete: complete
      });
    }
  }], [{
    key: "create",
    value: function create(observerCallback) {
      return new Observable(observerCallback);
    }
  }]);

  return Observable;
}();

var Subject =
/*#__PURE__*/
function () {
  function Subject() {
    _classCallCheck(this, Subject);

    this.next = this.next.bind(this);
    this.error = this.error.bind(this);
    this.complete = this.complete.bind(this);
    this.observers = [];
  }

  _createClass(Subject, [{
    key: "subscribe",
    value: function subscribe(observer) {
      var subscription = Subscription.createSimple(observer);
      this.observers.push(subscription.observer);
      return subscription;
    }
  }, {
    key: "unsubscribe",
    value: function unsubscribe() {
      this.cleanup(function (observer) {
        observer.cleanup();
        return false;
      });
    }
  }, {
    key: "cleanup",
    value: function cleanup(callback) {
      this.observers = this.observers.filter(function (observer) {
        if (!observer.isComplete) {
          var ret = callback(observer);

          if (typeof ret === 'boolean') {
            return ret;
          }

          return true;
        }

        return false;
      });
      return this;
    }
  }, {
    key: "next",
    value: function next(value) {
      this.cleanup(function (observer) {
        observer.onNext(value);
      });
    }
  }, {
    key: "error",
    value: function error() {
      for (var _len = arguments.length, errors = new Array(_len), _key = 0; _key < _len; _key++) {
        errors[_key] = arguments[_key];
      }

      this.cleanup(function (observer) {
        observer.onError.apply(observer, errors);
      });
    }
  }, {
    key: "complete",
    value: function complete() {
      this.cleanup(function (observer) {
        observer.onComplete();
      });
    }
  }]);

  return Subject;
}();

/**
 * Makes an observable "hot" -- very useful for things like DOM event listeners
 * so that the events do not get bound numerous amounts of times
 *
 * @param {Observable} cold$
 * @returns {Observable}
 */

var makeHot = function makeHot(cold$) {
  var subject = new Subject();
  var mainSub = cold$.subscribe(subject);
  var refs = 0;
  return new Observable(function (observer) {
    refs++; // if the main subscription is complete we have to resubscribe to it

    if (mainSub.isComplete) {
      mainSub = cold$.subscribe(subject);
    }

    var sub = subject.subscribe(observer);
    return function () {
      refs--;
      if (refs === 0) mainSub.unsubscribe();
      sub.unsubscribe();
    };
  });
};

Observable.prototype.makeHot = function () {
  return makeHot(this);
};

/**
 * @private
 * @param source$
 * @param nextFunction
 * @param dispose
 * @returns {Observable}
 */

var passThroughNext = function passThroughNext(source$, nextFunction) {
  var _this = this;

  var dispose = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : noop;
  return new Observable(function (observer) {
    var subscription = source$.subscribe({
      next: nextFunction.bind(_this, observer),
      error: observer.error,
      complete: observer.complete
    });
    return function () {
      dispose();
      subscription.unsubscribe();
    };
  });
};

/**
 * Will map each value to a new value using the callback
 *
 * @param {Observable} source$
 * @param {Function} mapCallback
 * @returns {Observable}
 */

var map = function map(source$, mapCallback) {
  return passThroughNext(source$, function (_ref, value) {
    var next = _ref.next;
    next(mapCallback(value));
  });
};
Observable.map = map;

Observable.prototype.map = function (mapCallback) {
  return map(this, mapCallback);
};

/**
 * Will filter out values that you do not want to subscribe to
 *
 * @param {Observable} source$
 * @param {Function} filterCallback
 * @returns {Observable}
 */

var filter = function filter(source$, filterCallback) {
  return passThroughNext(source$, function (_ref, value) {
    var next = _ref.next;

    if (filterCallback(value)) {
      next(value);
    }
  });
};
Observable.filter = filter;

Observable.prototype.filter = function (filterCallback) {
  return filter(this, filterCallback);
};

/**
 * Will delay output from the observable until a specific time interval has passed
 *
 * @param {Observable} source$
 * @param {Number} time amount of time in milliseconds
 * @returns {Observable}
 */

var delay = function delay(source$, time) {
  return new Observable(function (observer) {
    var subscription;
    var timerId = setTimeout(function () {
      subscription = source$.subscribe(observer.next, observer.error, observer.complete);
    }, time);
    return function () {
      clearTimeout(timerId);

      if (subscription) {
        subscription.unsubscribe();
      }
    };
  });
};
Observable.delay = delay;

Observable.prototype.delay = function (time) {
  return delay(this, time);
};

/**
 * Will run some callback before passing the current value to the subscription
 *
 * @param {Observable} source$
 * @param {Function} runCallback
 * @returns {Observable}
 *
 * @example
 * doStuff(obs$, (value) => console.log(value))
 *  .subscribe((sameValue) => console.log('Will log the same value: ', sameValue))l
 */

var doStuff = function doStuff(source$, runCallback) {
  return passThroughNext(source$, function (_ref, value) {
    var next = _ref.next;
    runCallback(value);
    next(value);
  });
};
Observable.do = doStuff;

Observable.prototype.do = function (runCallback) {
  return doStuff(this, runCallback);
};

/**
 * Debounces values that will be sent down the stream.
 * Will only output values if there has not been any new values in the past time interval passed
 *
 * @param {Observable} source$
 * @param {Number} time amount of time in milliseconds
 * @returns {Observable}
 */

var debounceTime = function debounceTime(source$, time) {
  return new Observable(function (observer) {
    var timerId;
    var subscription = passThroughNext(source$, function (_ref, value) {
      var next = _ref.next;
      clearTimeout(timerId);
      timerId = setTimeout(function () {
        next(value);
      }, time);
    }, function () {
      clearTimeout(timerId);
    }).subscribe(observer);
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.debounceTime = debounceTime;

Observable.prototype.debounceTime = function (time) {
  return debounceTime(this, time);
};

/**
 * Takes a number of values that satisfy the `filterCallback` then completes
 *
 * @param {Observable} source$
 * @param {Number} amount
 * @param {Function} [filterCallback]
 * @returns {Observable}
 */

var take = function take(source$, amount) {
  var filterCallback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {
    return true;
  };
  return new Observable(function (observer) {
    var taken = [];
    var subscription = passThroughNext(source$, function (_ref, value) {
      var next = _ref.next,
          complete = _ref.complete;
      var isComplete = taken.length === amount;

      if (!isComplete && filterCallback(value)) {
        taken.push(value);
        next(value);
      }

      if (isComplete) {
        complete();
      }
    }).subscribe(observer);
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.take = take;

Observable.prototype.take = function (amount, filterCallback) {
  return take(this, amount, filterCallback);
};

/**
 * Takes values from the `source$` until the `takeSource$` emits one value
 *
 * @param {Observable} source$
 * @param {Observable} takeSource$
 * @returns {Observable}
 */

var takeUntil = function takeUntil(source$, takeSource$) {
  return new Observable(function (observer) {
    var subscription = source$.subscribe(observer);
    var takeSubscription = takeSource$.subscribe(observer.complete, observer.error, observer.complete);
    return function () {
      takeSubscription.unsubscribe();
      subscription.unsubscribe();
    };
  });
};
Observable.takeUntil = takeUntil;

Observable.prototype.takeUntil = function (takeSource$) {
  return takeUntil(this, takeSource$);
};

/**
 * Will take the first value from the observable and then complete
 * This is an alias for `take(obs$, 1, callback)`
 *
 * @param {Observable} source$
 * @param {Function} [filterCallback] filter out values before taking the first one
 */

var first = function first(source$, filterCallback) {
  return take(source$, 1, filterCallback);
};
Observable.first = first;

Observable.prototype.first = function (filterCallback) {
  return first(this, filterCallback);
};

var global$1 = (typeof global !== "undefined" ? global :
            typeof self !== "undefined" ? self :
            typeof window !== "undefined" ? window : {});

/**
 * Reverse of [`fromPromise`](#frompromise)
 * <br />
 * Converts an Observable to a promise, takes the first value and then completes
 *
 * @param {Observable} source$
 * @returns {Promise<*>}
 */

var toPromise = function toPromise(source$) {
  if (!global$1.Promise) {
    throw new Error('Promise does not exist in the current context');
  }

  return new Promise(function (respond, reject) {
    return first(source$).subscribe({
      next: respond,
      error: reject,
      complete: respond
    });
  });
};
Observable.toPromise = toPromise;

Observable.prototype.toPromise = function () {
  return toPromise(this);
};

/**
 * Takes any number of arguments and outputs them in order in an Observable stream
 *
 * @param {*[]} args
 * @returns {Observable}
 */

var of = function of() {
  for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return new Observable(function (observer) {
    Array.from(args).forEach(function (value) {
      observer.next(value);
    });
    observer.complete();
  });
};
Observable.of = of;

/**
 * @private
 * @param source$
 * @param mapCallback
 * @returns {Observable}
 */

var passThroughNextObservable = function passThroughNextObservable(source$, mapCallback) {
  return passThroughNext(source$, function (_ref, value) {
    var next = _ref.next;
    var nextValue$ = mapCallback(value);

    if (!isObservable(nextValue$)) {
      nextValue$ = of(nextValue$);
    }

    next(nextValue$);
  });
};

/**
 * @private
 * @param subscriptions
 * @param complete
 * @returns {*}
 */
var onSubscriptionsComplete = function onSubscriptionsComplete(subscriptions, complete) {
  var allComplete = subscriptions.filter(function (s) {
    return !s.isComplete;
  }).length <= 0;

  if (allComplete) {
    return complete();
  }
};

/**
 * Same as `map(obs$, mapCallback)` but will take the value of the callback and turn it from an observable to a value
 *
 * @param {Observable} source$
 * @param {Function} mapCallback
 * @returns {Observable}
 */

var flatMap = function flatMap(source$, mapCallback) {
  return new Observable(function (observer) {
    var subscription = {
      isComplete: false
    };
    var nextSubscriptionList = [];

    var onComplete = function onComplete() {
      return onSubscriptionsComplete([subscription].concat(nextSubscriptionList), observer.complete);
    };

    subscription = passThroughNextObservable(source$, mapCallback).subscribe(function (nextValue$) {
      var nextSubscription = nextValue$.subscribe(observer.next, observer.error, onComplete);
      nextSubscriptionList.push(nextSubscription);
    }, observer.error, onComplete);
    return function () {
      nextSubscriptionList.forEach(function (nextSub) {
        return nextSub.unsubscribe();
      });
      subscription.unsubscribe();
    };
  });
};
Observable.flatMap = flatMap;

Observable.prototype.flatMap = function (mapCallback) {
  return flatMap(this, mapCallback);
};

/**
 * The value from the mapCallback is an observable and if another value comes through the previous observable is cancelled
 * <br />
 * This is useful for things like typeaheads where you dont want a request for every keypress
 *
 * @param {Observable} source$
 * @param {Function} mapCallback
 * @returns {Observable}
 */

var switchMap = function switchMap(source$, mapCallback) {
  return new Observable(function (observer) {
    var currentSubscription;
    var nextSubscription;
    currentSubscription = passThroughNextObservable(source$, mapCallback).subscribe(function (nextValue$) {
      // unsubscribe to the previous subscription if a new value comes in
      if (nextSubscription) {
        nextSubscription.unsubscribe();
      }

      nextSubscription = nextValue$.subscribe(function (value) {
        observer.next(value);
      }, observer.error, observer.complete);
    }, observer.error, observer.complete);
    return function () {
      if (nextSubscription) {
        nextSubscription.unsubscribe();
      }

      currentSubscription.unsubscribe();
    };
  });
};
Observable.switchMap = switchMap;

Observable.prototype.switchMap = function (mapCallback) {
  return switchMap(this, mapCallback);
};

/**
 * Will merge any number of observables into one observable stream
 *
 * @param {Observable[]} sources$
 * @returns {Observable}
 */

var merge = function merge(sources$) {
  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var subscriptions = [];

    var onComplete = function onComplete() {
      return onSubscriptionsComplete(subscriptions, complete);
    };

    var subscribeTo = function subscribeTo(obs$) {
      return obs$.subscribe({
        next: next,
        error: error,
        complete: onComplete
      });
    };

    subscriptions = sources$.map(function (s$, index) {
      return subscribeTo(s$, index);
    });
    return function () {
      return subscriptions.forEach(function (s) {
        return s.unsubscribe();
      });
    };
  });
};
Observable.merge = merge;

Observable.prototype.merge = function (otherSources$) {
  return merge([this].concat(_toConsumableArray(otherSources$)));
};

/**
 * @param {Observable} source$
 * @param {Function} scanCallback
 * @param {*} [startValue]
 * @returns {Observable}
 */

var scan = function scan(source$, scanCallback, startValue) {
  return new Observable(function (_ref) {
    var _next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var previousValue = startValue;
    var subscription = source$.subscribe({
      next: function next(value) {
        previousValue = scanCallback(previousValue, value);

        _next(previousValue);
      },
      error: error,
      complete: complete
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.scan = scan;

Observable.prototype.scan = function (scanCallback, startValue) {
  return scan(this, scanCallback, startValue);
};

/**
 * _Sort of_ the same way that Array.reduce works, it will concatenate all of the values
 * passing through an Observable event stream with a given scanCallback
 *
 * @param {Observable} source$
 * @param {Function} scanCallback
 * @param {*} [startValue]
 * @returns {Observable}
 */

var reduce = function reduce(source$, scanCallback, startValue) {
  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var lastValue = 0;
    var subscription = scan(source$, scanCallback, startValue).subscribe({
      next: function next(value) {
        lastValue = value;
      },
      error: error,
      complete: function complete() {
        next(lastValue);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.reduce = reduce;

Observable.prototype.reduce = function (scanCallback, startValue) {
  return reduce(this, scanCallback, startValue);
};

/**
 * Concatenate any number of observables together
 *
 * @param {Observable} source$
 * @param {Observable} nextSource$
 * @param {Observable[]} otherSources$
 * @returns {Observable}
 */

var concat = function concat(source$, nextSource$) {
  for (var _len = arguments.length, otherSources$ = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    otherSources$[_key - 2] = arguments[_key];
  }

  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var innerSubscription;

    var completeSubscription = function completeSubscription() {
      if (nextSource$ && otherSources$.length) {
        innerSubscription = concat.apply(void 0, [nextSource$].concat(otherSources$)).subscribe({
          next: next,
          error: error,
          complete: complete
        });
      } else if (nextSource$) {
        innerSubscription = nextSource$.subscribe({
          next: next,
          error: error,
          complete: complete
        });
      } else {
        complete();
      }
    };

    var subscription = source$.subscribe({
      next: next,
      error: error,
      complete: function complete() {
        completeSubscription();
      }
    });
    return function () {
      if (innerSubscription) {
        innerSubscription.unsubscribe();
      }

      subscription.unsubscribe();
    };
  });
};
Observable.concat = concat;

Observable.prototype.concat = function () {
  for (var _len2 = arguments.length, args$ = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args$[_key2] = arguments[_key2];
  }

  return concat.apply(void 0, [this].concat(args$));
};

var startIndex = {
  values: [],
  indexAt: -1
};

var argsCallback = function argsCallback() {
  return Array.from(arguments);
};
/**
 * @param {Observable} sources$
 * @param {Function} combineCallback
 * @returns {Observable}
 */


var zip = function zip(sources$) {
  var combineCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : argsCallback;
  return new Observable(function (_ref) {
    var _next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var subscriptions = [];
    var latest = sources$.map(function (s$) {
      return JSON.parse(JSON.stringify(startIndex));
    });

    var checkAllHasValue = function checkAllHasValue(index) {
      return latest.reduce(function (current, _ref2) {
        var values = _ref2.values;
        return current && Object.keys(values).includes(index + '');
      }, true);
    };

    var getValuesAtIndex = function getValuesAtIndex(index) {
      return latest.map(function (_ref3) {
        var values = _ref3.values;
        return values[index];
      });
    };

    var onComplete = function onComplete() {
      return onSubscriptionsComplete(subscriptions, complete);
    };

    var subscribeTo = function subscribeTo(obs$, index) {
      return obs$.subscribe({
        next: function next(value) {
          var currentIndex = latest[index];
          currentIndex.values.push(value);
          currentIndex.indexAt += 1;
          var allHasValue = checkAllHasValue(currentIndex.indexAt);

          if (allHasValue) {
            var values = getValuesAtIndex(currentIndex.indexAt);

            _next(combineCallback.apply(void 0, _toConsumableArray(values)));
          }
        },
        error: error,
        complete: onComplete
      });
    };

    subscriptions = sources$.map(function (s$, index) {
      return subscribeTo(s$, index);
    });
    return function () {
      return subscriptions.forEach(function (s) {
        return s.unsubscribe();
      });
    };
  });
};
Observable.zip = zip;

Observable.prototype.zip = function (otherSources$, combineCallback) {
  return zip([this].concat(_toConsumableArray(otherSources$)), combineCallback);
};

var nullHash = void 0;

var argsCallback$1 = function argsCallback() {
  return Array.from(arguments);
};
/**
 * combines multiple observables at the same time.
 * it will only call the observer's next function when all observables have emitted at least one value
 *
 * @param {Observable[]} sources$
 * @param {Function} combineCallback
 * @returns {Observable}
 */


var combine = function combine(sources$) {
  var combineCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : argsCallback$1;
  return new Observable(function (_ref) {
    var _next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var subscriptions = [];
    var latest = sources$.map(function (s$) {
      return nullHash;
    });
    var allHasValue = false;

    var checkAllHasValue = function checkAllHasValue() {
      return latest.filter(function (l) {
        return l == nullHash;
      }).length <= 0;
    };

    var onComplete = function onComplete() {
      return onSubscriptionsComplete(subscriptions, complete);
    };

    var subscribeTo = function subscribeTo(obs$, index) {
      return obs$.subscribe({
        next: function next(value) {
          latest[index] = value;
          allHasValue = allHasValue || checkAllHasValue();

          if (allHasValue) {
            _next(combineCallback.apply(void 0, _toConsumableArray(latest)));
          }
        },
        error: error,
        complete: onComplete
      });
    };

    subscriptions = sources$.map(function (s$, index) {
      return subscribeTo(s$, index);
    });
    return function () {
      return subscriptions.forEach(function (s) {
        return s.unsubscribe();
      });
    };
  });
};
Observable.combine = combine;

Observable.prototype.combine = function (otherSources$, combineCallback) {
  return combine([this].concat(_toConsumableArray(otherSources$)), combineCallback);
};

/**
 * Counts the number of values coming into the stream on complete
 *
 * @param {Observable} source$
 * @param {Function} [countCallback] a function to use to filter out values that you do not wish to count
 * @returns {Observable}
 */

var count = function count(source$) {
  var countCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (value) {
    return true;
  };
  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var count = 0;
    var subscription = filter(source$, countCallback).subscribe({
      next: function next() {
        count++;
      },
      error: error,
      complete: function complete() {
        next(count);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.count = count;

Observable.prototype.count = function (countCallback) {
  return count(this, countCallback);
};

/**
 * Sums all the values of an observable upon completion
 *
 * @param {Observable} source$
 */

var sum = function sum(source$) {
  return reduce(source$, function (a, b) {
    return a + b;
  });
};
Observable.sum = sum;

Observable.prototype.sum = function () {
  return sum(this);
};

/**
 * Will result in the maximum value passed to the observer then will complete
 *
 * @param {Observable} source$
 * @returns {Observable}
 */

var max = function max(source$) {
  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var maxNumber;
    var subscription = source$.subscribe({
      next: function next(value) {
        if (!maxNumber || value > maxNumber) {
          maxNumber = value;
        }
      },
      error: error,
      complete: function complete() {
        next(maxNumber);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.max = max;

Observable.prototype.max = function () {
  return max(this);
};

/**
 * Will result in the minimum value passed to the observer then will complete
 *
 * @param source$
 * @returns {Observable}
 */

var min = function min(source$) {
  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var minNumber;
    var subscription = source$.subscribe({
      next: function next(value) {
        if (!minNumber || value < minNumber) {
          minNumber = value;
        }
      },
      error: error,
      complete: function complete() {
        next(minNumber);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.min = min;

Observable.prototype.min = function () {
  return min(this);
};

/**
 * Takes an average of everything coming into the event stream until it completes
 *
 * @param {Observable} source$
 * @returns {Observable}
 */

var average = function average(source$) {
  return new Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var count = 0;
    var total = 0;
    var subscription = source$.subscribe({
      next: function next(value) {
        count++;
        total += value;
      },
      error: error,
      complete: function complete() {
        next(total / count);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};
Observable.average = average;

Observable.prototype.average = function () {
  return average(this);
};

/**
 * @private
 * @param {XMLHttpRequest} xhr
 * @returns {*}
 */

function checkStatus(xhr) {
  if (xhr.status >= 200 && xhr.status < 300) {
    return Object.assign(xhr, {
      toJSON: function toJSON() {
        return JSON.parse(xhr.responseText);
      }
    });
  } else {
    var error = new Error(xhr.statusText);
    error.xhr = xhr;
    throw error;
  }
}
/**
 * Make a cancellable XHR request
 *
 * @param {string} url
 * @param {string} [method]
 * @param {*} [requestData]
 * @param {{name: string, value: string}[]} [headers]
 *
 * @returns {Observable}
 */


var ajax = function ajax(url) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$method = _ref.method,
      method = _ref$method === void 0 ? 'GET' : _ref$method,
      _ref$requestData = _ref.requestData,
      requestData = _ref$requestData === void 0 ? '' : _ref$requestData,
      _ref$headers = _ref.headers,
      headers = _ref$headers === void 0 ? [] : _ref$headers;

  return new Observable(function (observer) {
    var xhr = new XMLHttpRequest();

    function xhrStateChanged() {
      if (xhr.readyState === XMLHttpRequest.DONE) {
        try {
          observer.next(checkStatus(xhr));
        } catch (error) {
          observer.error(error);
        }

        observer.complete();
      }
    }

    xhr.open(method, url);
    headers.forEach(function (header) {
      xhr.setRequestHeader(header.name, header.value);
    });
    xhr.onreadystatechange = xhrStateChanged;
    xhr.send(JSON.stringify(requestData));
    return function () {
      return xhr.abort();
    };
  });
};
Observable.ajax = ajax;

/**
 * Will create an observable that will listen for an event on a DOM element
 *
 * Utilizes the [`makeHot`](#makehot) method so that the event does not get reattached on every subscription
 *
 * @param {string} eventName
 * @param {Element} element
 * @param {Function} [mapCallback]
 * @returns {Observable}
 */

var fromEvent = function fromEvent(eventName, element) {
  var mapCallback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function (v) {
    return v;
  };
  return makeHot(new Observable(function (_ref) {
    var next = _ref.next;

    var listener = function listener(event) {
      next(mapCallback(event));
    };

    element.addEventListener(eventName, listener, false);
    return function () {
      element.removeEventListener(eventName, listener, false);
    };
  }));
};
Observable.fromEvent = fromEvent;

/**
 * Turns a promise into an observable that emits the value of the promise and then completes
 *
 * @param {Promise} promise
 * @returns {Observable}
 */

var fromPromise = function fromPromise(promise) {
  return new Observable(function (_ref) {
    var next = _ref.next,
        complete = _ref.complete,
        error = _ref.error;

    if (isPromise(promise)) {
      promise.then(function (values) {
        next(values);
        complete();
      }, error).catch(error);
    } else {
      error('Passed an invalid object to fromPromise', promise);
    }
  });
};
Observable.fromPromise = fromPromise;

/**
 * Output a range of numbers to an Observable stream
 *
 * @param {Number} [start]
 * @param {Number} end
 * @returns {Observable}
 */

var range = function range() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$start = _ref.start,
      start = _ref$start === void 0 ? 0 : _ref$start,
      end = _ref.end;

  return new Observable(function (_ref2) {
    var next = _ref2.next,
        complete = _ref2.complete;

    for (var current = start; current <= end; ++current) {
      next(current);
    }

    complete();
  });
};
Observable.range = range;

/**
 * Creates an interval that will count on every interval tick
 *
 * @param {Number} time in milliseconds to output the interval count
 * @param {Number} [start] number to start the interval count at
 * @returns {Observable}
 */

var interval = function interval(time) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  return new Observable(function (_ref) {
    var next = _ref.next;
    var count = start;
    var id = setInterval(function () {
      var nextNumber = count++;
      next(nextNumber);
    }, time);
    return function () {
      return clearInterval(id);
    };
  });
};
Observable.interval = interval;

exports.Subscription = Subscription;
exports.Observable = Observable;
exports.Subject = Subject;
exports.Observer = Observer;
exports.noop = noop;
exports.isPromise = isPromise;
exports.isObservable = isObservable;
exports.makeHot = makeHot;
exports.map = map;
exports.filter = filter;
exports.delay = delay;
exports.doStuff = doStuff;
exports.debounceTime = debounceTime;
exports.take = take;
exports.takeUntil = takeUntil;
exports.first = first;
exports.toPromise = toPromise;
exports.flatMap = flatMap;
exports.switchMap = switchMap;
exports.merge = merge;
exports.scan = scan;
exports.reduce = reduce;
exports.concat = concat;
exports.zip = zip;
exports.combine = combine;
exports.count = count;
exports.sum = sum;
exports.max = max;
exports.min = min;
exports.average = average;
exports.ajax = ajax;
exports.fromEvent = fromEvent;
exports.fromPromise = fromPromise;
exports.range = range;
exports.interval = interval;
exports.of = of;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.js.map
