"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Observable = void 0;

var _Subscription = require("./Subscription");

var _utilities = require("./utilities");

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * Creates a new Observable
 * @param {Function} observerCallback
 *
 * @example
 * const obs$ = new Observable((observer) => {
 *   for (let i = 0; i < 10; ++i) {
 *     observer.next(i);
 *   }
 *
 *   observer.complete();
 * });
 *
 * @example
 * const obs$ = Observable.create((observer) => {
 *   for (let i = 0; i < 10; ++i) {
 *     observer.next(i);
 *   }
 *
 *   observer.complete();
 * });
 */
var Observable =
/*#__PURE__*/
function () {
  function Observable(observerCallback) {
    _classCallCheck(this, Observable);

    this.observerCallback = observerCallback;
  }
  /**
   * @param {Function} observerCallback
   * @returns {Observable}
   */


  _createClass(Observable, [{
    key: "subscribe",

    /**
     * Subscribe to an observable stream
     *
     * @param {{ next: Function, error: Function, complete: Function }} next
     * @param {null} error
     * @param {null} complete
     * @returns {Subscription}
     *
     * @example
     * observable$.subscribe({
     *   next(value) {
     *     console.log('logs if next', value);
     *   },
     *   error() {
     *     console.log('logs if error');
     *   },
     *   complete() {
     *     console.log('logs if complete');
     *   }
     * });
     */

    /**
     *
     * @param {Function} [next]
     * @param {Function} [error]
     * @param {Function} [complete]
     * @returns {Subscription}
     *
     * @example
     * observable$.subscribe(
     *  (value) => console.log('logs if next', value),
     *  () => console.log('logs if error'),
     *  () => console.log('logs if complete')
     * );
     */
    value: function subscribe() {
      var next = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _utilities.noop;
      var error = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _utilities.noop;
      var complete = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _utilities.noop;
      return new _Subscription.Subscription(this.observerCallback, {
        next: next,
        error: error,
        complete: complete
      });
    }
  }], [{
    key: "create",
    value: function create(observerCallback) {
      return new Observable(observerCallback);
    }
  }]);

  return Observable;
}();

exports.Observable = Observable;