"use strict";

var _Observable = require("./Observable");

var _Subscription = require("./Subscription");

describe('Observable', function () {
  test('new Observable will give you a new Observable instance', function () {
    expect(new _Observable.Observable()).toBeInstanceOf(_Observable.Observable);
    expect(_Observable.Observable.create()).toBeInstanceOf(_Observable.Observable);
  });
  test('observerCallback is the same as the function passed to the constructor', function () {
    var callback = function callback() {
      return null;
    };

    var obs$ = new _Observable.Observable(callback);
    expect(obs$.observerCallback).toBe(callback);
  });
  test('subscribe returns a subscription object', function () {
    var obs$ = new _Observable.Observable();
    expect(obs$.subscribe()).toBeInstanceOf(_Subscription.Subscription);
  });
  test('the observerCallback will not be called when subscribe is never called', function () {
    expect.assertions(0);
    new _Observable.Observable(function () {
      expect(true).toBeTruthy();
    });
  });
  test('the observerCallback will be called when subscribe is called once', function () {
    expect.assertions(1);
    var obs$ = new _Observable.Observable(function () {
      expect(true).toBeTruthy();
    });
    obs$.subscribe();
  });
  test('the error callback will get hit if observerCallback throws an error', function () {
    expect.assertions(2);
    var obs$ = new _Observable.Observable(function () {
      expect(true).toBeTruthy();
      throw new Error('I have an error');
    });
    obs$.subscribe({
      error: function error() {
        expect(true).toBeTruthy();
      }
    });
  });
  test('the error callback will receive the error thrown by the observerCallback', function () {
    expect.assertions(2);

    var _error2 = new Error('I have an error');

    var obs$ = new _Observable.Observable(function () {
      expect(true).toBeTruthy();
      throw _error2;
    });
    obs$.subscribe({
      error: function error(_error) {
        expect(_error).toBe(_error2);
      }
    });
  });
});