"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Observer = void 0;

var _utilities = require("./utilities");

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * @param {{next: Function, error: Function, complete: Function}} observer
 */
var Observer =
/*#__PURE__*/
function () {
  function Observer() {
    var observer = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Observer);

    this.isComplete = false;
    this.dispose = _utilities.noop;
    this.setupObserver(observer);
  }
  /**
   * Cleans up and disposes of all events in the stream
   *
   * @returns {Observer}
   */


  _createClass(Observer, [{
    key: "cleanup",
    value: function cleanup() {
      this.isComplete = true;
      this.dispose();
      this.setupObserverObject = {
        next: _utilities.noop,
        error: _utilities.noop,
        complete: _utilities.noop
      };
      this.dispose = _utilities.noop;
      return this;
    }
  }, {
    key: "catchErrors",
    value: function catchErrors(callback) {
      var _this = this;

      return function () {
        try {
          return callback.apply(void 0, arguments);
        } catch (errors) {
          _this.onError(errors);
        }
      };
    }
    /**
     * @param {Function} callback
     * @returns {Function}
     */

  }, {
    key: "use",
    value: function use(callback) {
      var _this2 = this;

      var callbackCatch = this.catchErrors(callback);
      var response = callbackCatch({
        next: function next(value) {
          return _this2.onNext(value);
        },
        error: function error() {
          return _this2.onError.apply(_this2, arguments);
        },
        complete: function complete() {
          return _this2.onComplete();
        }
      });

      if (typeof response === 'function') {
        this.dispose = this.catchErrors(response);
      } else {
        this.dispose = _utilities.noop;
      }

      return this.dispose;
    }
  }, {
    key: "onNext",
    value: function onNext(value) {
      if (this.isComplete) {
        return null;
      }

      return this.setupObserverObject.next(value);
    }
  }, {
    key: "onError",
    value: function onError() {
      var _this$setupObserverOb;

      if (this.isComplete) {
        return null;
      }

      return (_this$setupObserverOb = this.setupObserverObject).error.apply(_this$setupObserverOb, arguments);
    }
  }, {
    key: "onComplete",
    value: function onComplete() {
      if (this.isComplete) {
        return null;
      }

      this.cleanup();
      return this.setupObserverObject.complete();
    }
  }, {
    key: "setupObserver",
    value: function setupObserver() {
      var setupObserverObject = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
        next: _utilities.noop,
        error: _utilities.noop,
        complete: _utilities.noop
      };

      // assumes that an object was passed as first value to subscription
      if (typeof setupObserverObject.next !== 'function' && _typeof(setupObserverObject.next) === 'object') {
        return this.setupObserver(setupObserverObject.next);
      }

      var _setupObserverObject$ = setupObserverObject.next,
          next = _setupObserverObject$ === void 0 ? _utilities.noop : _setupObserverObject$,
          _setupObserverObject$2 = setupObserverObject.error,
          error = _setupObserverObject$2 === void 0 ? _utilities.noop : _setupObserverObject$2,
          _setupObserverObject$3 = setupObserverObject.complete,
          complete = _setupObserverObject$3 === void 0 ? _utilities.noop : _setupObserverObject$3;
      this.setupObserverObject = {
        next: this.catchErrors(next),
        error: error,
        complete: this.catchErrors(complete)
      };
      return this;
    }
  }]);

  return Observer;
}();

exports.Observer = Observer;