"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ajax = void 0;

var _Observable = require("../Observable");

/**
 * @private
 * @param {XMLHttpRequest} xhr
 * @returns {*}
 */
function checkStatus(xhr) {
  if (xhr.status >= 200 && xhr.status < 300) {
    return Object.assign(xhr, {
      toJSON: function toJSON() {
        return JSON.parse(xhr.responseText);
      }
    });
  } else {
    var error = new Error(xhr.statusText);
    error.xhr = xhr;
    throw error;
  }
}
/**
 * Make a cancellable XHR request
 *
 * @param {string} url
 * @param {string} [method]
 * @param {*} [requestData]
 * @param {{name: string, value: string}[]} [headers]
 *
 * @returns {Observable}
 */


var ajax = function ajax(url) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$method = _ref.method,
      method = _ref$method === void 0 ? 'GET' : _ref$method,
      _ref$requestData = _ref.requestData,
      requestData = _ref$requestData === void 0 ? '' : _ref$requestData,
      _ref$headers = _ref.headers,
      headers = _ref$headers === void 0 ? [] : _ref$headers;

  return new _Observable.Observable(function (observer) {
    var xhr = new XMLHttpRequest();

    function xhrStateChanged() {
      if (xhr.readyState === XMLHttpRequest.DONE) {
        try {
          observer.next(checkStatus(xhr));
        } catch (error) {
          observer.error(error);
        }

        observer.complete();
      }
    }

    xhr.open(method, url);
    headers.forEach(function (header) {
      xhr.setRequestHeader(header.name, header.value);
    });
    xhr.onreadystatechange = xhrStateChanged;
    xhr.send(JSON.stringify(requestData));
    return function () {
      return xhr.abort();
    };
  });
};

exports.ajax = ajax;
_Observable.Observable.ajax = ajax;