"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fromEvent = void 0;

var _Observable = require("../Observable");

var _makeHot = require("../utilities/makeHot");

/**
 * Will create an observable that will listen for an event on a DOM element
 *
 * Utilizes the [`makeHot`](#makehot) method so that the event does not get reattached on every subscription
 *
 * @param {string} eventName
 * @param {Element} element
 * @param {Function} [mapCallback]
 * @returns {Observable}
 */
var fromEvent = function fromEvent(eventName, element) {
  var mapCallback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function (v) {
    return v;
  };
  return (0, _makeHot.makeHot)(new _Observable.Observable(function (_ref) {
    var next = _ref.next;

    var listener = function listener(event) {
      next(mapCallback(event));
    };

    element.addEventListener(eventName, listener, false);
    return function () {
      element.removeEventListener(eventName, listener, false);
    };
  }));
};

exports.fromEvent = fromEvent;
_Observable.Observable.fromEvent = fromEvent;