"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fromPromise = void 0;

var _Observable = require("../Observable");

var _utilities = require("../utilities");

/**
 * Turns a promise into an observable that emits the value of the promise and then completes
 *
 * @param {Promise} promise
 * @returns {Observable}
 */
var fromPromise = function fromPromise(promise) {
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        complete = _ref.complete,
        error = _ref.error;

    if ((0, _utilities.isPromise)(promise)) {
      promise.then(function (values) {
        next(values);
        complete();
      }, error).catch(error);
    } else {
      error('Passed an invalid object to fromPromise', promise);
    }
  });
};

exports.fromPromise = fromPromise;
_Observable.Observable.fromPromise = fromPromise;