"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.interval = void 0;

var _Observable = require("../Observable");

/**
 * Creates an interval that will count on every interval tick
 *
 * @param {Number} time in milliseconds to output the interval count
 * @param {Number} [start] number to start the interval count at
 * @returns {Observable}
 */
var interval = function interval(time) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next;
    var count = start;
    var id = setInterval(function () {
      var nextNumber = count++;
      next(nextNumber);
    }, time);
    return function () {
      return clearInterval(id);
    };
  });
};

exports.interval = interval;
_Observable.Observable.interval = interval;