"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.average = void 0;

var _Observable = require("../Observable");

/**
 * Takes an average of everything coming into the event stream until it completes
 *
 * @param {Observable} source$
 * @returns {Observable}
 */
var average = function average(source$) {
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var count = 0;
    var total = 0;
    var subscription = source$.subscribe({
      next: function next(value) {
        count++;
        total += value;
      },
      error: error,
      complete: function complete() {
        next(total / count);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.average = average;
_Observable.Observable.average = average;

_Observable.Observable.prototype.average = function () {
  return average(this);
};