"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.combine = void 0;

var _Observable = require("../Observable");

var _onSubscriptionsComplete = require("../utilities/onSubscriptionsComplete");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var nullHash = void 0;

var argsCallback = function argsCallback() {
  return Array.from(arguments);
};
/**
 * combines multiple observables at the same time.
 * it will only call the observer's next function when all observables have emitted at least one value
 *
 * @param {Observable[]} sources$
 * @param {Function} combineCallback
 * @returns {Observable}
 */


var combine = function combine(sources$) {
  var combineCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : argsCallback;
  return new _Observable.Observable(function (_ref) {
    var _next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var subscriptions = [];
    var latest = sources$.map(function (s$) {
      return nullHash;
    });
    var allHasValue = false;

    var checkAllHasValue = function checkAllHasValue() {
      return latest.filter(function (l) {
        return l == nullHash;
      }).length <= 0;
    };

    var onComplete = function onComplete() {
      return (0, _onSubscriptionsComplete.onSubscriptionsComplete)(subscriptions, complete);
    };

    var subscribeTo = function subscribeTo(obs$, index) {
      return obs$.subscribe({
        next: function next(value) {
          latest[index] = value;
          allHasValue = allHasValue || checkAllHasValue();

          if (allHasValue) {
            _next(combineCallback.apply(void 0, _toConsumableArray(latest)));
          }
        },
        error: error,
        complete: onComplete
      });
    };

    subscriptions = sources$.map(function (s$, index) {
      return subscribeTo(s$, index);
    });
    return function () {
      return subscriptions.forEach(function (s) {
        return s.unsubscribe();
      });
    };
  });
};

exports.combine = combine;
_Observable.Observable.combine = combine;

_Observable.Observable.prototype.combine = function (otherSources$, combineCallback) {
  return combine([this].concat(_toConsumableArray(otherSources$)), combineCallback);
};