"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.concat = void 0;

var _Observable = require("../Observable");

/**
 * Concatenate any number of observables together
 *
 * @param {Observable} source$
 * @param {Observable} nextSource$
 * @param {Observable[]} otherSources$
 * @returns {Observable}
 */
var concat = function concat(source$, nextSource$) {
  for (var _len = arguments.length, otherSources$ = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    otherSources$[_key - 2] = arguments[_key];
  }

  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var innerSubscription;

    var completeSubscription = function completeSubscription() {
      if (nextSource$ && otherSources$.length) {
        innerSubscription = concat.apply(void 0, [nextSource$].concat(otherSources$)).subscribe({
          next: next,
          error: error,
          complete: complete
        });
      } else if (nextSource$) {
        innerSubscription = nextSource$.subscribe({
          next: next,
          error: error,
          complete: complete
        });
      } else {
        complete();
      }
    };

    var subscription = source$.subscribe({
      next: next,
      error: error,
      complete: function complete() {
        completeSubscription();
      }
    });
    return function () {
      if (innerSubscription) {
        innerSubscription.unsubscribe();
      }

      subscription.unsubscribe();
    };
  });
};

exports.concat = concat;
_Observable.Observable.concat = concat;

_Observable.Observable.prototype.concat = function () {
  for (var _len2 = arguments.length, args$ = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    args$[_key2] = arguments[_key2];
  }

  return concat.apply(void 0, [this].concat(args$));
};