"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.count = void 0;

var _Observable = require("../Observable");

var _filter = require("./filter");

/**
 * Counts the number of values coming into the stream on complete
 *
 * @param {Observable} source$
 * @param {Function} [countCallback] a function to use to filter out values that you do not wish to count
 * @returns {Observable}
 */
var count = function count(source$) {
  var countCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (value) {
    return true;
  };
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var count = 0;
    var subscription = (0, _filter.filter)(source$, countCallback).subscribe({
      next: function next() {
        count++;
      },
      error: error,
      complete: function complete() {
        next(count);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.count = count;
_Observable.Observable.count = count;

_Observable.Observable.prototype.count = function (countCallback) {
  return count(this, countCallback);
};