"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.debounceTime = void 0;

var _Observable = require("../Observable");

var _passThroughNext = require("./passThroughNext");

/**
 * Debounces values that will be sent down the stream.
 * Will only output values if there has not been any new values in the past time interval passed
 *
 * @param {Observable} source$
 * @param {Number} time amount of time in milliseconds
 * @returns {Observable}
 */
var debounceTime = function debounceTime(source$, time) {
  return new _Observable.Observable(function (observer) {
    var timerId;
    var subscription = (0, _passThroughNext.passThroughNext)(source$, function (_ref, value) {
      var next = _ref.next;
      clearTimeout(timerId);
      timerId = setTimeout(function () {
        next(value);
      }, time);
    }, function () {
      clearTimeout(timerId);
    }).subscribe(observer);
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.debounceTime = debounceTime;
_Observable.Observable.debounceTime = debounceTime;

_Observable.Observable.prototype.debounceTime = function (time) {
  return debounceTime(this, time);
};