"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.delay = void 0;

var _Observable = require("../Observable");

/**
 * Will delay output from the observable until a specific time interval has passed
 *
 * @param {Observable} source$
 * @param {Number} time amount of time in milliseconds
 * @returns {Observable}
 */
var delay = function delay(source$, time) {
  return new _Observable.Observable(function (observer) {
    var subscription;
    var timerId = setTimeout(function () {
      subscription = source$.subscribe(observer.next, observer.error, observer.complete);
    }, time);
    return function () {
      clearTimeout(timerId);

      if (subscription) {
        subscription.unsubscribe();
      }
    };
  });
};

exports.delay = delay;
_Observable.Observable.delay = delay;

_Observable.Observable.prototype.delay = function (time) {
  return delay(this, time);
};