"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doStuff = void 0;

var _Observable = require("../Observable");

var _passThroughNext = require("./passThroughNext");

/**
 * Will run some callback before passing the current value to the subscription
 *
 * @param {Observable} source$
 * @param {Function} runCallback
 * @returns {Observable}
 *
 * @example
 * doStuff(obs$, (value) => console.log(value))
 *  .subscribe((sameValue) => console.log('Will log the same value: ', sameValue))l
 */
var doStuff = function doStuff(source$, runCallback) {
  return (0, _passThroughNext.passThroughNext)(source$, function (_ref, value) {
    var next = _ref.next;
    runCallback(value);
    next(value);
  });
};

exports.doStuff = doStuff;
_Observable.Observable.do = doStuff;

_Observable.Observable.prototype.do = function (runCallback) {
  return doStuff(this, runCallback);
};