"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.flatMap = void 0;

var _Observable = require("../Observable");

var _passThroughNextObservable = require("./passThroughNextObservable");

var _onSubscriptionsComplete = require("../utilities/onSubscriptionsComplete");

/**
 * Same as `map(obs$, mapCallback)` but will take the value of the callback and turn it from an observable to a value
 *
 * @param {Observable} source$
 * @param {Function} mapCallback
 * @returns {Observable}
 */
var flatMap = function flatMap(source$, mapCallback) {
  return new _Observable.Observable(function (observer) {
    var subscription = {
      isComplete: false
    };
    var nextSubscriptionList = [];

    var onComplete = function onComplete() {
      return (0, _onSubscriptionsComplete.onSubscriptionsComplete)([subscription].concat(nextSubscriptionList), observer.complete);
    };

    subscription = (0, _passThroughNextObservable.passThroughNextObservable)(source$, mapCallback).subscribe(function (nextValue$) {
      var nextSubscription = nextValue$.subscribe(observer.next, observer.error, onComplete);
      nextSubscriptionList.push(nextSubscription);
    }, observer.error, onComplete);
    return function () {
      nextSubscriptionList.forEach(function (nextSub) {
        return nextSub.unsubscribe();
      });
      subscription.unsubscribe();
    };
  });
};

exports.flatMap = flatMap;
_Observable.Observable.flatMap = flatMap;

_Observable.Observable.prototype.flatMap = function (mapCallback) {
  return flatMap(this, mapCallback);
};