"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.max = void 0;

var _Observable = require("../Observable");

/**
 * Will result in the maximum value passed to the observer then will complete
 *
 * @param {Observable} source$
 * @returns {Observable}
 */
var max = function max(source$) {
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var maxNumber;
    var subscription = source$.subscribe({
      next: function next(value) {
        if (!maxNumber || value > maxNumber) {
          maxNumber = value;
        }
      },
      error: error,
      complete: function complete() {
        next(maxNumber);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.max = max;
_Observable.Observable.max = max;

_Observable.Observable.prototype.max = function () {
  return max(this);
};