"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.merge = void 0;

var _Observable = require("../Observable");

var _onSubscriptionsComplete = require("../utilities/onSubscriptionsComplete");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

/**
 * Will merge any number of observables into one observable stream
 *
 * @param {Observable[]} sources$
 * @returns {Observable}
 */
var merge = function merge(sources$) {
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var subscriptions = [];

    var onComplete = function onComplete() {
      return (0, _onSubscriptionsComplete.onSubscriptionsComplete)(subscriptions, complete);
    };

    var subscribeTo = function subscribeTo(obs$) {
      return obs$.subscribe({
        next: next,
        error: error,
        complete: onComplete
      });
    };

    subscriptions = sources$.map(function (s$, index) {
      return subscribeTo(s$, index);
    });
    return function () {
      return subscriptions.forEach(function (s) {
        return s.unsubscribe();
      });
    };
  });
};

exports.merge = merge;
_Observable.Observable.merge = merge;

_Observable.Observable.prototype.merge = function (otherSources$) {
  return merge([this].concat(_toConsumableArray(otherSources$)));
};