"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.passThroughNext = void 0;

var _Observable = require("../Observable");

var _utilities = require("../utilities");

/**
 * @private
 * @param source$
 * @param nextFunction
 * @param dispose
 * @returns {Observable}
 */
var passThroughNext = function passThroughNext(source$, nextFunction) {
  var _this = this;

  var dispose = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _utilities.noop;
  return new _Observable.Observable(function (observer) {
    var subscription = source$.subscribe({
      next: nextFunction.bind(_this, observer),
      error: observer.error,
      complete: observer.complete
    });
    return function () {
      dispose();
      subscription.unsubscribe();
    };
  });
};

exports.passThroughNext = passThroughNext;