"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reduce = void 0;

var _Observable = require("../Observable");

var _scan = require("./scan");

/**
 * _Sort of_ the same way that Array.reduce works, it will concatenate all of the values
 * passing through an Observable event stream with a given scanCallback
 *
 * @param {Observable} source$
 * @param {Function} scanCallback
 * @param {*} [startValue]
 * @returns {Observable}
 */
var reduce = function reduce(source$, scanCallback, startValue) {
  return new _Observable.Observable(function (_ref) {
    var next = _ref.next,
        error = _ref.error,
        _complete = _ref.complete;
    var lastValue = 0;
    var subscription = (0, _scan.scan)(source$, scanCallback, startValue).subscribe({
      next: function next(value) {
        lastValue = value;
      },
      error: error,
      complete: function complete() {
        next(lastValue);

        _complete();
      }
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.reduce = reduce;
_Observable.Observable.reduce = reduce;

_Observable.Observable.prototype.reduce = function (scanCallback, startValue) {
  return reduce(this, scanCallback, startValue);
};