"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.scan = void 0;

var _Observable = require("../Observable");

/**
 * @param {Observable} source$
 * @param {Function} scanCallback
 * @param {*} [startValue]
 * @returns {Observable}
 */
var scan = function scan(source$, scanCallback, startValue) {
  return new _Observable.Observable(function (_ref) {
    var _next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var previousValue = startValue;
    var subscription = source$.subscribe({
      next: function next(value) {
        previousValue = scanCallback(previousValue, value);

        _next(previousValue);
      },
      error: error,
      complete: complete
    });
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.scan = scan;
_Observable.Observable.scan = scan;

_Observable.Observable.prototype.scan = function (scanCallback, startValue) {
  return scan(this, scanCallback, startValue);
};