"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.switchMap = void 0;

var _Observable = require("../Observable");

var _passThroughNextObservable = require("./passThroughNextObservable");

/**
 * The value from the mapCallback is an observable and if another value comes through the previous observable is cancelled
 * <br />
 * This is useful for things like typeaheads where you dont want a request for every keypress
 *
 * @param {Observable} source$
 * @param {Function} mapCallback
 * @returns {Observable}
 */
var switchMap = function switchMap(source$, mapCallback) {
  return new _Observable.Observable(function (observer) {
    var currentSubscription;
    var nextSubscription;
    currentSubscription = (0, _passThroughNextObservable.passThroughNextObservable)(source$, mapCallback).subscribe(function (nextValue$) {
      // unsubscribe to the previous subscription if a new value comes in
      if (nextSubscription) {
        nextSubscription.unsubscribe();
      }

      nextSubscription = nextValue$.subscribe(function (value) {
        observer.next(value);
      }, observer.error, observer.complete);
    }, observer.error, observer.complete);
    return function () {
      if (nextSubscription) {
        nextSubscription.unsubscribe();
      }

      currentSubscription.unsubscribe();
    };
  });
};

exports.switchMap = switchMap;
_Observable.Observable.switchMap = switchMap;

_Observable.Observable.prototype.switchMap = function (mapCallback) {
  return switchMap(this, mapCallback);
};