"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.take = void 0;

var _Observable = require("../Observable");

var _passThroughNext = require("./passThroughNext");

/**
 * Takes a number of values that satisfy the `filterCallback` then completes
 *
 * @param {Observable} source$
 * @param {Number} amount
 * @param {Function} [filterCallback]
 * @returns {Observable}
 */
var take = function take(source$, amount) {
  var filterCallback = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function () {
    return true;
  };
  return new _Observable.Observable(function (observer) {
    var taken = [];
    var subscription = (0, _passThroughNext.passThroughNext)(source$, function (_ref, value) {
      var next = _ref.next,
          complete = _ref.complete;
      var isComplete = taken.length === amount;

      if (!isComplete && filterCallback(value)) {
        taken.push(value);
        next(value);
      }

      if (isComplete) {
        complete();
      }
    }).subscribe(observer);
    return function () {
      return subscription.unsubscribe();
    };
  });
};

exports.take = take;
_Observable.Observable.take = take;

_Observable.Observable.prototype.take = function (amount, filterCallback) {
  return take(this, amount, filterCallback);
};