"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.takeUntil = void 0;

var _Observable = require("../Observable");

/**
 * Takes values from the `source$` until the `takeSource$` emits one value
 *
 * @param {Observable} source$
 * @param {Observable} takeSource$
 * @returns {Observable}
 */
var takeUntil = function takeUntil(source$, takeSource$) {
  return new _Observable.Observable(function (observer) {
    var subscription = source$.subscribe(observer);
    var takeSubscription = takeSource$.subscribe(observer.complete, observer.error, observer.complete);
    return function () {
      takeSubscription.unsubscribe();
      subscription.unsubscribe();
    };
  });
};

exports.takeUntil = takeUntil;
_Observable.Observable.takeUntil = takeUntil;

_Observable.Observable.prototype.takeUntil = function (takeSource$) {
  return takeUntil(this, takeSource$);
};