"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toPromise = void 0;

var _Observable = require("../Observable");

var _first = require("./first");

/**
 * Reverse of [`fromPromise`](#frompromise)
 * <br />
 * Converts an Observable to a promise, takes the first value and then completes
 *
 * @param {Observable} source$
 * @returns {Promise<*>}
 */
var toPromise = function toPromise(source$) {
  if (!global.Promise) {
    throw new Error('Promise does not exist in the current context');
  }

  return new Promise(function (respond, reject) {
    return (0, _first.first)(source$).subscribe({
      next: respond,
      error: reject,
      complete: respond
    });
  });
};

exports.toPromise = toPromise;
_Observable.Observable.toPromise = toPromise;

_Observable.Observable.prototype.toPromise = function () {
  return toPromise(this);
};