"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.zip = void 0;

var _Observable = require("../Observable");

var _onSubscriptionsComplete = require("../utilities/onSubscriptionsComplete");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var startIndex = {
  values: [],
  indexAt: -1
};

var argsCallback = function argsCallback() {
  return Array.from(arguments);
};
/**
 * @param {Observable} sources$
 * @param {Function} combineCallback
 * @returns {Observable}
 */


var zip = function zip(sources$) {
  var combineCallback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : argsCallback;
  return new _Observable.Observable(function (_ref) {
    var _next = _ref.next,
        error = _ref.error,
        complete = _ref.complete;
    var subscriptions = [];
    var latest = sources$.map(function (s$) {
      return JSON.parse(JSON.stringify(startIndex));
    });

    var checkAllHasValue = function checkAllHasValue(index) {
      return latest.reduce(function (current, _ref2) {
        var values = _ref2.values;
        return current && Object.keys(values).includes(index + '');
      }, true);
    };

    var getValuesAtIndex = function getValuesAtIndex(index) {
      return latest.map(function (_ref3) {
        var values = _ref3.values;
        return values[index];
      });
    };

    var onComplete = function onComplete() {
      return (0, _onSubscriptionsComplete.onSubscriptionsComplete)(subscriptions, complete);
    };

    var subscribeTo = function subscribeTo(obs$, index) {
      return obs$.subscribe({
        next: function next(value) {
          var currentIndex = latest[index];
          currentIndex.values.push(value);
          currentIndex.indexAt += 1;
          var allHasValue = checkAllHasValue(currentIndex.indexAt);

          if (allHasValue) {
            var values = getValuesAtIndex(currentIndex.indexAt);

            _next(combineCallback.apply(void 0, _toConsumableArray(values)));
          }
        },
        error: error,
        complete: onComplete
      });
    };

    subscriptions = sources$.map(function (s$, index) {
      return subscribeTo(s$, index);
    });
    return function () {
      return subscriptions.forEach(function (s) {
        return s.unsubscribe();
      });
    };
  });
};

exports.zip = zip;
_Observable.Observable.zip = zip;

_Observable.Observable.prototype.zip = function (otherSources$, combineCallback) {
  return zip([this].concat(_toConsumableArray(otherSources$)), combineCallback);
};