"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeHot = void 0;

var _Observable = require("../Observable");

var _Subject = require("../Subject");

/**
 * Makes an observable "hot" -- very useful for things like DOM event listeners
 * so that the events do not get bound numerous amounts of times
 *
 * @param {Observable} cold$
 * @returns {Observable}
 */
var makeHot = function makeHot(cold$) {
  var subject = new _Subject.Subject();
  var mainSub = cold$.subscribe(subject);
  var refs = 0;
  return new _Observable.Observable(function (observer) {
    refs++; // if the main subscription is complete we have to resubscribe to it

    if (mainSub.isComplete) {
      mainSub = cold$.subscribe(subject);
    }

    var sub = subject.subscribe(observer);
    return function () {
      refs--;
      if (refs === 0) mainSub.unsubscribe();
      sub.unsubscribe();
    };
  });
};

exports.makeHot = makeHot;

_Observable.Observable.prototype.makeHot = function () {
  return makeHot(this);
};